/* 
 * Applet to draw a horizontal bar chart for a week, based on parameters 
 * Kimberly Nicholls 
 *
 * Based on 
 * @(#)Chart.java	1.6 95/03/27 Sami Shaio
*/

/*
 */

/* 
  $Log: WeekChart.java,v $
  Revision 1.7  1996/08/08 01:14:07  kimberly
  layout and copy stuff

  Revision 1.6  1996/08/07 18:54:18  kimberly
  change blue to match button bar

  Revision 1.5  1996/07/31 03:34:17  kimberly
  add double buffering to prevent flashing

  Revision 1.4  1996/07/27 01:37:15  kimberly
  minor fixes for beta

  Revision 1.3  1996/07/26 18:34:27  kimberly
  move total labels to bottom and center

  Revision 1.2  1996/07/26 16:29:12  kimberly
  change font; add full_day parameter to determine where red starts

  Revision 1.1  1996/07/26 16:04:14  kimberly
  initial revision: converted from Chart.java

*/

package Tock;
import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.io.*;
import java.lang.*;

public class WeekChart extends Applet {

  static final int	titleHeight = 15;
  static final int	columns = 7;
  static final int	scale = 5;
  static final int	barSpacing = 10;

  String		title;
  Font		        titleFont;
  FontMetrics		titleFontMetrics;

  float			values[];
  String		labels[];
  String		dates[];

  int			maxLabelWidth = 0;
  int			barWidth;
  float			full_day = 8;
  float                 sum = 0;

  Image        ofsImg;
  Graphics     ofsGraphics;

  public synchronized void init() {
    String rs;

    titleFont = new Font("Helvetica", Font.BOLD, 10);
    titleFontMetrics = getFontMetrics(titleFont);
    title = "Hours This Week";

    values = new float[columns];
    labels = new String[columns];
    dates = new String[columns];
    labels[0] = "Sun";
    labels[1] = "Mon";
    labels[2] = "Tue";
    labels[3] = "Wed";
    labels[4] = "Thu";
    labels[5] = "Fri";
    labels[6] = "Sat";

    rs = getParameter("full_day");
    if (rs != null) {
      try {
	Float f = new Float(rs);
	full_day = f.floatValue();
      } catch (NumberFormatException e) {
	full_day = (float) 8;
      }
    }

    for (int i=0; i < columns; i++) {
      // parse the value for this column
      rs = getParameter("db" + i);
      if (rs != null) {
	try {
	  Float f = new Float(rs);
	  values[i] = f.floatValue();
	} catch (NumberFormatException e) {
	  values[i] = (float)0;
	}
      }

      rs = getParameter("d" + i);
      if (rs != null) {
	try {
	  dates[i] = rs.substring(0,rs.lastIndexOf("/"));
	  if (titleFontMetrics.stringWidth(dates[i].toString()) > 
	      maxLabelWidth)
	    maxLabelWidth = titleFontMetrics.stringWidth(dates[i].toString());

	} catch (StringIndexOutOfBoundsException e) {
	  dates[i] = "";
	}
      }
    }

    barWidth = maxLabelWidth;
    resize(Math.max(columns * (barWidth + barSpacing),
		    titleFontMetrics.stringWidth(title)) + 
	   titleFont.getSize() + 5, (int) ((2 * titleFont.getSize()) + 5 + 
					   titleFont.getSize()));

  }

  // clear the title and redraw with current sum
  synchronized void drawTitle(Graphics g) {
    int i;

    // calculate the total
    sum = 0;
    for (i = 0; i < columns; i++)  sum += values[i];

    // clear the title
    g.setColor(Color.white);
    g.fillRect(0,1+size().height - 
	       (titleFontMetrics.getDescent()+titleFont.getSize()),
	       size().width,size().height);

    // draw the new title
    g.setColor(Color.black);
    i = titleFontMetrics.stringWidth(title+": "+sum);
    g.setFont(titleFont);
    g.drawString(title+": "+sum, Math.max((size().width - i)/2, 0),
		 size().height - titleFontMetrics.getDescent()); 
  }

  // draw one bar
  // parameters: 
  //   g         graphics context
  //   barNum    which bar to draw (0-6)
  //   barVal    value for bar
  //   cx, cy    x,y to draw from
  synchronized void drawBar(Graphics g, int barNum, float barVal, 
	int cx, int cy) {

    // clear the current bar and total
    g.setColor(Color.white);
    g.fillRect(cx,0,barWidth+3,size().height - 2 -
	4 * (titleFont.getSize() - titleFontMetrics.getDescent()));

    // draw the shadow
    g.setColor(Color.black);		
    g.fillRect(cx + 3, cy - 3, barWidth, (int) (values[barNum] * scale));

    // draw the bar with values > full_day in a different color
    if (values[barNum] > (float) full_day) {
      g.setColor(Color.red);
      g.fillRect(cx, cy, barWidth, 
		 (int) ((values[barNum]-full_day) * scale));
      g.setColor(new Color(153,204,204));
      g.fillRect(cx, (int) (cy+((values[barNum]-full_day) * scale)), 
		 barWidth, (int) (full_day * scale));
    }
    else {
      g.setColor(new Color(153,204,204));
      g.fillRect(cx, cy, barWidth, (int) (values[barNum] * scale));
    }

    // draw the total centered under the bar
    g.setFont(titleFont);
    g.drawString("" + values[barNum], cx + 
	((barWidth - titleFontMetrics.stringWidth(""+values[barNum]))/2), 
	size().height - 4 - 
	4 * (titleFont.getSize() - titleFontMetrics.getDescent()));
    
    drawTitle(g);
  }

  // set the bar value for barNum and recalculate the sum
  // does not call update - this must be called to actually draw the
  // new value
  // this makes it possible to minimize the number of updates
  public synchronized void updateBar(int barNum, float barVal) {
    Graphics g;
    g = this.getGraphics();
    int cx,cy;

    values[barNum] = barVal;
    sum = 0;
    for (int i = 0; i < columns; i++)  sum += values[i];
  }

  public synchronized void paint(Graphics g) {
    update(g);
  }

  // overrides default update: uses double buffering to prevent flashing
  public synchronized void update(Graphics g) {
    int i, j;
    char l[] = new char[1];
    int cx,cy;

    // create off screen image
    if (ofsGraphics == null) {
      ofsImg = createImage(size().width, size().height);
      ofsGraphics = ofsImg.getGraphics();
    }

    // clear off screen image
    ofsGraphics.setColor(Color.white);
    ofsGraphics.fillRect(0,0,235,100);

    // draw on to off screen image
    drawTitle(ofsGraphics);
    for (i=0; i < columns; i++) {
      // set the next X coordinate to account for the label
      // being wider than the bar width.
      cx = (Math.max((barWidth + barSpacing),maxLabelWidth) *  i) + 
	barSpacing;
      
      // center the bar chart
      cx += Math.max((size().width - (columns *
				      (barWidth + (2 * barSpacing))))/2,0);
      // set the next Y coordinate to account for the height
      // of the bar as well as the title and labels painted
      // at the bottom of the chart.
      cy = (int) (size().height - (values[i] * scale) - 3 - 
		  (4 * titleFont.getSize()));

      // draw the label
      ofsGraphics.setColor(Color.black);		
      ofsGraphics.drawString((String)labels[i], cx, size().height - 8 - 
	   (2 * (titleFont.getSize() - titleFontMetrics.getDescent())));

      // draw the date
      ofsGraphics.setColor(Color.gray);
      ofsGraphics.drawString((String)dates[i], cx, size().height - 5 -
		   (titleFont.getSize() - titleFontMetrics.getDescent()));

      // draw the bar
      drawBar(ofsGraphics,i,values[i],cx,cy);
    }
    // paint the off screen image onto the real screen
    g.drawImage(ofsImg,0,0,this);
  }
}



