
<!-- application: 	Tock 	-->
<!-- version: 		1.0 					-->
<!-- company name:	Internet Media Services 		-->
<!-- filename:		db_down.html		 		-->
<!-- description:	helper functions for writing reports	-->

<!-- ===================================================================== -->

/* 
  $Log: reports.js,v $
  Revision 1.17  1996/08/22 00:52:23  kimberly
  user can click on any part of entry to edit it

  Revision 1.16  1996/08/21 17:54:14  kimberly
  rename index.html to index.html; break long lines; add comments

  Revision 1.15  1996/08/09 02:05:06  kimberly
  change version number

  Revision 1.14  1996/08/09 02:03:04  kimberly
  add commenting

  Revision 1.13  1996/08/09 00:04:26  kimberly
  add instructions on how to edit an entry

  Revision 1.12  1996/08/08 01:14:04  kimberly
  layout and copy stuff

  Revision 1.11  1996/08/02 21:37:00  kimberly
  fix typo

  Revision 1.10  1996/08/02 19:50:48  kimberly
  remove debugging info and shorten lines

  Revision 1.9  1996/08/02 02:20:06  kimberly
  changes for flattened directory s structure

  Revision 1.8  1996/07/31 17:53:27  kimberly
  use writeln to make generated HTML easier to read

  Revision 1.7  1996/07/31 03:41:54  kimberly
  change date ranges to always include today; replace last month report with month to date

  Revision 1.6  1996/07/29 19:06:05  kimberly
  use formatHours to print numbers with 2 decimal places

  Revision 1.5  1996/07/27 01:37:17  kimberly
  minor fixes for beta

  Revision 1.4  1996/07/26 18:06:58  kimberly
  use select month(date) as mon... form for splitting dates because query.date.getMonth seemed to stop working (crashed the debugger)

  Revision 1.3  1996/07/25 00:19:37  kimberly
  use formatNumber function

  Revision 1.2  1996/07/23 18:53:16  kimberly
  Add Log tag

*/

<!-- ===================================================================== -->
<!-- object for holding hour totals -->
function totalObj() {
  this.billable = 0;
  this.non_billable = 0;
  this.item_total = 0;
}

<!-- ===================================================================== -->

<!-- writes table header for reports					  -->
<!-- parameters:							  -->
<!--   report_type   the label for the left column (Project,Employee, etc) -->
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
<!-- called from list_hours.html and do_report.html		   -->
function writeTableHeader(report_type,multi_hrs_col) {
  writeln("<table border=1>");
  writeln("<tr bgcolor=\"#bbbbbb\">");
  if (multi_hrs_col) {
    writeln("<th rowspan=2>"+report_type+"</th>");
    writeln("<th colspan=3>Hours</th>");
    writeln("</tr>");
    writeln("<tr bgcolor=\"#bbbbbb\">");
    writeln("<th>Billable</th>");
    writeln("<th>Non Bill</th>");
    writeln("<th>Total</th>");
  }
  else {
    writeln("<th>"+report_type+"</th>");
    writeln("<th>Hours</th>");
  }
  writeln("</tr>");
}

<!-- ===================================================================== -->

<!-- write one hours record and increment totals 			-->
<!-- parameters:							-->
<!--   rec_num		current record number				-->
<!--   num_hours	number of hours in current record		-->
<!--   billable		whether hours are billable			-->
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
<!--   total_obj	object containing totals for report		--> 
<!-- returns new rec_num						-->
<!-- called by writeHours and writeDateReport			        -->
function writeHoursRec(rec_num,num_hours,billable,multi_hrs_col,total_obj) {
  if (rec_num == 0 && billable == "N" && multi_hrs_col) {
    <!-- there weren't any billable hours -->
    writeln("<td align=right>0.00</td>");
    writeln("<td align=right>"+formatHours(num_hours)+"</td>");
    rec_num++;
  }
  else 
    writeln("<td align=right>"+formatHours(num_hours)+"</td>");
  if (billable == "Y") 
    total_obj.billable += num_hours;
  if (billable == "N") 
    total_obj.non_billable += num_hours;
  total_obj.item_total += num_hours;
  rec_num++;

  return rec_num;
}

<!-- ===================================================================== -->

<!-- write total for one item.  fill in missing fields, if any, with 0	-->
<!-- parameters:							-->
<!--   rec_num		current record number				-->
<!--   item_total	total for current item				-->
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
<!-- called by writeHours and writeDateReport			        -->
function writeItemTotal(rec_num,item_total,multi_hrs_col) {

  if (rec_num == 0) {  
    <!-- no records found at all -->
    writeln("<td align=right>0.00</td>");
    if (multi_hrs_col)
      writeln("<td align=right>0.00</td>");
  }
  if (rec_num == 1 && multi_hrs_col) 
    <!-- only billable record found -->
    writeln("<td align=right>0.00</td>");

  if (multi_hrs_col) 
    writeln("<td align=right>"+formatHours(item_total)+"</td>");
  writeln("</tr>");
}

<!-- ===================================================================== -->

<!-- write the contents of the total object -->
<!-- parameters:							-->
<!--   total_obj	object containing totals for report		--> 
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
<!-- called by writeReport and writeDateReport			        -->
function writeTotal(total_obj,multi_hrs_col) {

  writeln("<tr>");
  writeln("<td bgcolor=\"#99cccc\"><b>Total</b></td>");
  if (multi_hrs_col) {
    writeln("<td align=right bgcolor=\"#99cccc\"><b>"+
	formatHours(total_obj.billable)+"</b></td>");
    writeln("<td align=right bgcolor=\"#99cccc\"><b>"+
	formatHours(total_obj.non_billable)+
          "</b></td>");
  }
  writeln("<td align=right bgcolor=\"#99cccc\"><b>"+formatHours(eval(
        total_obj.non_billable+total_obj.billable))+"</b></td>");
  writeln("</tr>");
}

<!-- ===================================================================== -->

<!-- select hours and billable for an item, then print the data and total  -->
<!-- parameters:							   -->
<!--   item_name	label for this table row			-->
<!--   query_string	database query which must select sum(hours) and -->
<!--			billable for the item
<!--   total_obj	object containing totals for report		--> 
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
function writeHours(item_name, query_string, total_obj, multi_hrs_col) {
  var rec_num = 0;
  var item_total = 0;
  var query_cursor;

  writeln("<tr>");
  writeln("<td>"+item_name+"</td>");
  query_cursor = database.cursor(query_string);
  while (query_cursor.next()) {
    rec_num = writeHoursRec(rec_num,query_cursor[0],query_cursor.billable,
              multi_hrs_col,total_obj);
  }

  writeItemTotal(rec_num,total_obj.item_total,multi_hrs_col);
  total_obj.item_total = 0;

  query_cursor.close();
}

<!-- ===================================================================== -->

<!-- select items, then call writeHours to write the hours for each item.  -->
<!-- then write final total for items.					   -->
<!-- parameters:							   -->
<!--    query_string	 a query that selects the id and name of an entity  -->
<!--			 (for example, select id,name from group).          -->
<!--    sub_query_string uses the id field of the query to select the sum   -->
<!--			 hours and whether the hours are billable from the  -->
<!--		   	 hours table.  The subquery should have the field   -->
<!--		         that matches the id in the outer query as its last -->
<!--			 element, because the value comes from the outer    -->
<!-- 			 query cursor. 	                                    -->
<!--   date_range	 date range in db query format			    -->
<!--   multi_hrs_col    whether report has billable/non billable cols	-->
<!-- called from list_hours.html and do_report.html		-->
function writeReport(query_string, sub_query_string, date_range, 
		     multi_hrs_col) {
  var total_obj = new totalObj();
  var query_cursor;

  query_cursor = database.cursor(query_string);
  while (query_cursor.next()) {
    writeHours(query_cursor[1],sub_query_string+query_cursor[0]+" and "+
	date_range+" group by billable order by billable desc",total_obj,
	multi_hrs_col);
  }
  query_cursor.close();

  writeTotal(total_obj,multi_hrs_col);
}

<!-- ===================================================================== -->

<!-- this is basically the same as writeReport. LiveWire retrieves dates  -->
<!-- like Mon Jul 01 01:00:00 PDT 1996, but the database date field is a  -->
<!-- date (no time info).  Let Informix do the joins so that date fields  -->
<!-- will have a consistent format.                                       --> 
<!-- don't do billable/non-billable because I can't tell when the date    -->
<!-- has changed:							  -->
<!--   prev_date = "xxx";						  -->
<!--   while (query_cursor.next() {					  -->
<!--     if (prev_date != query_cursor.date)  -- always true!		  -->
<!--       do a total 							  -->
<!--     ...								  -->
<!--     prev_date = query_cursor.date					  -->
<!--   }								  -->
<!-- parameters:							  -->
<!--   from_add		tables that need to be added to the from clause   -->
<!--   where_add	conditions that need to added to the where clause -->
<!--   date_range	 date range in db query format			  -->
function writeDateReport(from_add,where_add,date_range) {
  var total_obj = new totalObj();
  var query_cursor;
  var rec_num = 0;
  var first = true;

  query_cursor = database.cursor("select sum(hours),month(date) as mon, "+
	"day(date) as day, year(date) as yr,date from hours,project "+
	from_add+" where project_id=project.id and "+where_add+date_range+
	" group by date order by date asc");
  while (query_cursor.next()) {
    writeln("<tr>");
    writeln("<td>"+query_cursor.mon+"/"+
          query_cursor.day+"/"+query_cursor.yr+"</td>");
    rec_num = writeHoursRec(rec_num,query_cursor[0],"Y",false,total_obj);
    writeItemTotal(rec_num,total_obj.item_total,false);
    total_obj.item_total = 0;
    rec_num = 0;
  }
  query_cursor.close();

  writeTotal(total_obj,false);
}

<!-- ===================================================================== -->

<!-- write a report that contains each individual entry for a user  -->
function writeAllEntriesReport(user_id,date_range) {
  var total = 0;
  var query_cursor;
  var rec_num = 0;
  var first = true;

  // header
  if (client.id == user_id)
    writeln("To edit an entry, click on any field in the entry.");
  writeln("<P>");
  writeln("<table border=1>");
  writeln("<tr bgcolor=\"#bbbbbb\">");
  writeln("<th align=left>Date</th>");
  writeln("<th align=left>Hours</th>");
  writeln("<th align=left>Project</th>");
  writeln("<th align=left>Category</th>");
  writeln("<th align=left>Description</th>");
  writeln("<tr>");

  query_cursor = database.cursor("select date,hours,project.name,"+
      "category.name,description,hours.id from hours,project,category where "+
      date_range+" and user_id="+user_id+" and project_id=project.id and "+
      "category_id=category.id order by date,project.name,category.name");
  while (query_cursor.next()) {
    writeln("<tr>");
    if (client.id == user_id) {
      client.date_range = date_range;
      writeln("<td align=left><a href=\"edit_entry.html?id="+query_cursor.id+
	"\">"+(query_cursor.date.getMonth()+1)+"/"+
	query_cursor.date.getDate()+"/"+query_cursor.date.getYear()+
	"</a></td>");
      writeln("<td align=right><a href=\"edit_entry.html?id="+
	query_cursor.id+"\">"+formatHours(query_cursor.hours)+"</a></td>");
      writeln("<td nowrap align=left><a href=\"edit_entry.html?id="+
	query_cursor.id+"\">"+query_cursor[2]+"</a></td>");
      writeln("<td nowrap align=left><a href=\"edit_entry.html?id="+
	query_cursor.id+"\">"+query_cursor[3]+"</a></td>");
      writeln("<td align=left><a href=\"edit_entry.html?id="+
	query_cursor.id+"\">"+query_cursor.description+"</a></td>");
      }
    else {
      writeln("<td align=left>"+(query_cursor.date.getMonth()+1)+"/"+
	query_cursor.date.getDate()+"/"+query_cursor.date.getYear()+"</td>");
      writeln("<td align=right>"+formatHours(query_cursor.hours)+"</td>");
      writeln("<td nowrap align=left>"+query_cursor[2]+"</td>");
      writeln("<td nowrap align=left>"+query_cursor[3]+"</td>");
      writeln("<td align=left>"+query_cursor.description+"</td>");
    }
    writeln("<tr>");

    total += query_cursor.hours;
  }
  query_cursor.close();

  writeln("<tr>");
  writeln("<td bgcolor=\"#99cccc\"><b>Total</b></td>");
  writeln("<td align=right bgcolor=\"#99cccc\"><b>"+formatHours(total)+"</b></td>");
  writeln("</tr>");
  writeln("</table>");
}

<!-- ===================================================================== -->

<!-- convert a date range entered on a form to a database query date range -->
<!-- called from list_hours.html and do_report.html    	   -->
function parseReportDateRange() {
  var date_range;

  if (request.date_range_type == "user_range") 
    date_range = " date >= '"+request.from_date+"' and date <= '"+
	request.to_date+"'";
  else {
    if (request.date_range_select == "last week")
      date_range = " date <= today and date > today-7 "
    if (request.date_range_select == "last two weeks")
      date_range = " date <= today and date > today-14 "
    if (request.date_range_select == "last month") {
      today = new Date();
      year = today.getYear();
      month = today.getMonth()+1;
      day = 1;

      date_range = " date <= today and date >= '"+month+"-"+day+"-"+year+"' ";
      }
  }
  return date_range;
}

<!-- ===================================================================== -->
